@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -46,75 +46,59 @@
 static void spine_signal_handler(int spine_signal) {
 	signal(spine_signal, SIG_DFL);
 
-#if HAS_EXECINFO_H
-	// get void*'s for all entries on the stack
-	set.exit_size = backtrace(set.exit_stack, 10);
-#endif
-
 	set.exit_code = spine_signal;
 
 	switch (spine_signal) {
 		case SIGABRT:
-			spine_print_backtrace();
-			die("FATAL: Spine Interrupted by Abort Signal");
+			fprintf(stderr, "FATAL: Spine Interrupted by Abort Signal\n");
 			break;
 		case SIGINT:
-			spine_print_backtrace();
-			die("FATAL: Spine Interrupted by Console Operator");
+			fprintf(stderr, "FATAL: Spine Interrupted by Console Operator\n");
 			break;
 		case SIGSEGV:
-			spine_print_backtrace();
-			die("FATAL: Spine Encountered a Segmentation Fault");
+			fprintf(stderr, "FATAL: Spine Encountered a Segmentation Fault\n");
+
+			#ifdef HAVE_EXECINFO_H
+			int row = 0;
+			char **exit_strings = NULL;
+
+			fprintf(stderr, "Generating backtrace...%ld line(s)...\n", set.exit_size);
+
+			if (set.exit_size) {
+				set.exit_size = backtrace(set.exit_stack, set.exit_size);
+				exit_strings  = backtrace_symbols(set.exit_stack, set.exit_size);
+
+				if (exit_strings) {
+					for (row = 0; row < set.exit_size; row++) {
+						fprintf(stderr, "%3d: %s\n", row, exit_strings[row]);
+					}
+
+					free(exit_strings);
+				}
+			}
+			#endif
+
+			exit(1);
+
 			break;
 		case SIGBUS:
-			spine_print_backtrace();
-			die("FATAL: Spine Encountered a Bus Error");
+			fprintf(stderr, "FATAL: Spine Encountered a Bus Error\n");
 			break;
 		case SIGFPE:
-			spine_print_backtrace();
-			die("FATAL: Spine Encountered a Floating Point Exception");
+			fprintf(stderr, "FATAL: Spine Encountered a Floating Point Exception\n");
 			break;
 		case SIGQUIT:
-			spine_print_backtrace();
-			die("FATAL: Spine Encountered a Keyboard Quit Command");
+			fprintf(stderr, "FATAL: Spine Encountered a Keyboard Quit Command\n");
 			break;
 		case SIGPIPE:
-			spine_print_backtrace();
-			die("FATAL: Spine Encountered a Broken Pipe");
+			fprintf(stderr, "FATAL: Spine Encountered a Broken Pipe\n");
 			break;
 		default:
-			spine_print_backtrace();
-			die("FATAL: Spine Encountered An Unhandled Exception Signal Number: '%d'", spine_signal);
+			fprintf(stderr, "FATAL: Spine Encountered An Unhandled Exception Signal Number: '%d'\n", spine_signal);
 			break;
 	}
 }
 
-void spine_print_backtrace() {
-	int nptrs;
-	int bt_buf_size = 100;
-	void *buffer[bt_buf_size];
-	char **strings;
-	int j;
-
-	nptrs = backtrace(buffer, bt_buf_size);
-	fprintf(stderr, "backtrace() returned %d addresses\n", nptrs);
-
-	/* The call backtrace_symbols_fd(buffer, nptrs, STDOUT_FILENO)
-	 * would produce similar output to the following: */
-
-	strings = backtrace_symbols(buffer, nptrs);
-	if (strings == NULL) {
-		perror("Spine backtrace symbols follow");
-		exit(EXIT_FAILURE);
-	}
-
-	for (j = 0; j < nptrs; j++) {
-		fprintf(stderr, "%s\n", strings[j]);
-	}
-
-	free(strings);
-}
-
 static int spine_fatal_signals[] = {
 	SIGINT,
 	SIGPIPE,
