@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -160,7 +160,7 @@ int ping_host(host_t *host, ping_t *ping) {
  *
  */
 int ping_snmp(host_t *host, ping_t *ping) {
-	char *poll_result;
+	char *poll_result = NULL;
 	char *oid;
 	double begin_time, end_time, total_time;
 	double one_thousand = 1000.00;
@@ -276,7 +276,6 @@ int ping_icmp(host_t *host, ping_t *ping) {
 	struct   ip    *ip;
 	struct   icmp  *pkt;
 	unsigned char  *packet;
-	char     *new_hostname;
 
 	if (is_debug_device(host->id)) {
 		SPINE_LOG(("Device[%i] DEBUG: Entering ICMP Ping", host->id));
@@ -284,9 +283,6 @@ int ping_icmp(host_t *host, ping_t *ping) {
 		SPINE_LOG_DEBUG(("Device[%i] DEBUG: Entering ICMP Ping", host->id));
 	}
 
-	/* remove "tcp:" from hostname */
-	new_hostname = remove_tcp_udp_from_hostname(host->hostname);
-
 	/* get ICMP socket */
 	retry_count = 0;
 	while (TRUE) {
@@ -306,7 +302,6 @@ int ping_icmp(host_t *host, ping_t *ping) {
 			if (retry_count > 4) {
 				snprintf(ping->ping_response, SMALL_BUFSIZE, "ICMP: Ping unable to create ICMP Socket");
 				snprintf(ping->ping_status, 50, "down");
-				free(new_hostname);
 				#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
 				if (hasCaps() != TRUE) {
 					if (seteuid(getuid()) == -1) {
@@ -371,7 +366,7 @@ int ping_icmp(host_t *host, ping_t *ping) {
 		snprintf(ping->ping_response, SMALL_BUFSIZE, "default");
 
 		/* get address of hostname */
-		if (init_sockaddr(&fromname, new_hostname, 7)) {
+		if (init_sockaddr(&fromname, host->hostname, 7)) {
 			retry_count = 0;
 			total_time  = 0;
 			begin_time  = get_time_as_double();
@@ -384,16 +379,15 @@ int ping_icmp(host_t *host, ping_t *ping) {
 				if (retry_count > host->ping_retries) {
 					snprintf(ping->ping_response, SMALL_BUFSIZE, "ICMP: Ping timed out");
 					snprintf(ping->ping_status, 50, "down");
-					free(new_hostname);
 					free(packet);
 					close(icmp_socket);
 					return HOST_DOWN;
 				}
 
 				if (is_debug_device(host->id)) {
-					SPINE_LOG(("Device[%i] DEBUG: Attempting to ping %s, seq %d (Retry %d of %d)", host->id, new_hostname, icmp->icmp_seq, retry_count, host->ping_retries));
+					SPINE_LOG(("Device[%i] DEBUG: Attempting to ping %s, seq %d (Retry %d of %d)", host->id, host->hostname, icmp->icmp_seq, retry_count, host->ping_retries));
 				} else {
-					SPINE_LOG_DEBUG(("Device[%i] DEBUG: Attempting to ping %s, seq %d (Retry %d of %d)", host->id, new_hostname, icmp->icmp_seq, retry_count, host->ping_retries));
+					SPINE_LOG_DEBUG(("Device[%i] DEBUG: Attempting to ping %s, seq %d (Retry %d of %d)", host->id, host->hostname, icmp->icmp_seq, retry_count, host->ping_retries));
 				}
 
 				/* decrement the timeout value by the total time */
@@ -451,7 +445,6 @@ int ping_icmp(host_t *host, ping_t *ping) {
 								}
 								snprintf(ping->ping_response, SMALL_BUFSIZE, "ICMP: Device is Alive");
 								snprintf(ping->ping_status, 50, "%.5f", total_time);
-								free(new_hostname);
 								free(packet);
 								#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
 								if (hasCaps() != TRUE) {
@@ -503,7 +496,6 @@ int ping_icmp(host_t *host, ping_t *ping) {
 		} else {
 			snprintf(ping->ping_response, SMALL_BUFSIZE, "ICMP: Destination hostname invalid");
 			snprintf(ping->ping_status, 50, "down");
-			free(new_hostname);
 			free(packet);
 			#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
 			if (hasCaps() != TRUE) {
@@ -527,7 +519,6 @@ int ping_icmp(host_t *host, ping_t *ping) {
 	} else {
 		snprintf(ping->ping_response, SMALL_BUFSIZE, "ICMP: Destination address not specified");
 		snprintf(ping->ping_status, 50, "down");
-		free(new_hostname);
 		free(packet);
 		if (icmp_socket != -1) {
 			#if !(defined(__CYGWIN__) && !defined(SOLAR_PRIV))
@@ -577,7 +568,6 @@ int ping_udp(host_t *host, ping_t *ping) {
 	int    request_len;
 	int    return_code;
 	fd_set socket_fds;
-	char   *new_hostname;
 
 	if (is_debug_device(host->id)) {
 		SPINE_LOG(("Device[%i] DEBUG: Entering UDP Ping", host->id));
@@ -590,9 +580,6 @@ int ping_udp(host_t *host, ping_t *ping) {
 
 	begin_time = get_time_as_double();
 
-	/* remove "udp:" from hostname */
-	new_hostname = remove_tcp_udp_from_hostname(host->hostname);
-
 	/* convert the host timeout to a double precision number in seconds */
 	host_timeout = host->ping_timeout;
 
@@ -606,11 +593,10 @@ int ping_udp(host_t *host, ping_t *ping) {
 		snprintf(ping->ping_response, SMALL_BUFSIZE, "default");
 
 		/* get address of hostname */
-		if (init_sockaddr(&servername, new_hostname, host->ping_port)) {
+		if (init_sockaddr(&servername, host->hostname, host->ping_port)) {
 			if (connect(udp_socket, (struct sockaddr *) &servername, sizeof(servername)) < 0) {
 				snprintf(ping->ping_status, 50, "down");
 				snprintf(ping->ping_response, SMALL_BUFSIZE, "UDP: Cannot connect to host");
-				free(new_hostname);
 				close(udp_socket);
 				return HOST_DOWN;
 			}
@@ -629,7 +615,6 @@ int ping_udp(host_t *host, ping_t *ping) {
 				if (retry_count > host->ping_retries) {
 					snprintf(ping->ping_response, SMALL_BUFSIZE, "UDP: Ping timed out");
 					snprintf(ping->ping_status, 50, "down");
-					free(new_hostname);
 					close(udp_socket);
 					return HOST_DOWN;
 				}
@@ -679,7 +664,6 @@ int ping_udp(host_t *host, ping_t *ping) {
 							}
 							snprintf(ping->ping_response, SMALL_BUFSIZE, "UDP: Device is Alive");
 							snprintf(ping->ping_status, 50, "%.5f", total_time);
-							free(new_hostname);
 							close(udp_socket);
 							return HOST_UP;
 						}
@@ -692,7 +676,6 @@ int ping_udp(host_t *host, ping_t *ping) {
 					} else {
 						snprintf(ping->ping_response, SMALL_BUFSIZE, "UDP: Device is Down");
 						snprintf(ping->ping_status, 50, "%.5f", total_time);
-						free(new_hostname);
 						close(udp_socket);
 						return HOST_DOWN;
 					}
@@ -714,14 +697,12 @@ int ping_udp(host_t *host, ping_t *ping) {
 		} else {
 			snprintf(ping->ping_response, SMALL_BUFSIZE, "UDP: Destination hostname invalid");
 			snprintf(ping->ping_status, 50, "down");
-			free(new_hostname);
 			close(udp_socket);
 			return HOST_DOWN;
 		}
 	} else {
 		snprintf(ping->ping_response, SMALL_BUFSIZE, "UDP: Destination address invalid or unable to create socket");
 		snprintf(ping->ping_status, 50, "down");
-		free(new_hostname);
 		if (udp_socket != -1) close(udp_socket);
 		return HOST_DOWN;
 	}
@@ -749,7 +730,6 @@ int ping_tcp(host_t *host, ping_t *ping) {
 	struct sockaddr_in servername;
 	int    retry_count;
 	int    return_code;
-	char   *new_hostname;
 
 	if (is_debug_device(host->id)) {
 		SPINE_LOG(("Device[%i] DEBUG: Entering TCP Ping", host->id));
@@ -757,9 +737,6 @@ int ping_tcp(host_t *host, ping_t *ping) {
 		SPINE_LOG_DEBUG(("Device[%i] DEBUG: Entering TCP Ping", host->id));
 	}
 
-	/* remove "tcp:" from hostname */
-	new_hostname = remove_tcp_udp_from_hostname(host->hostname);
-
 	/* convert the host timeout to a double precision number in seconds */
 	host_timeout = host->ping_timeout;
 
@@ -779,7 +756,7 @@ int ping_tcp(host_t *host, ping_t *ping) {
 		snprintf(ping->ping_response, SMALL_BUFSIZE, "default");
 
 		/* get address of hostname */
-		if (init_sockaddr(&servername, new_hostname, host->ping_port)) {
+		if (init_sockaddr(&servername, host->hostname, host->ping_port)) {
 			/* first attempt a connect */
 			retry_count = 0;
 
@@ -809,21 +786,18 @@ int ping_tcp(host_t *host, ping_t *ping) {
 					}
 					snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Device is Alive");
 					snprintf(ping->ping_status, 50, "%.5f", total_time);
-					free(new_hostname);
 					close(tcp_socket);
 					return HOST_UP;
 				} else {
 					#if defined(__CYGWIN__)
 					snprintf(ping->ping_status, 50, "down");
 					snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Cannot connect to host");
-					free(new_hostname);
 					close(tcp_socket);
 					return HOST_DOWN;
 					#else
 					if (retry_count > host->ping_retries) {
 						snprintf(ping->ping_status, 50, "down");
 						snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Cannot connect to host");
-						free(new_hostname);
 						close(tcp_socket);
 						return HOST_DOWN;
 					} else {
@@ -835,14 +809,12 @@ int ping_tcp(host_t *host, ping_t *ping) {
 		} else {
 			snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Destination hostname invalid");
 			snprintf(ping->ping_status, 50, "down");
-			free(new_hostname);
 			close(tcp_socket);
 			return HOST_DOWN;
 		}
 	} else {
 		snprintf(ping->ping_response, SMALL_BUFSIZE, "TCP: Destination address invalid or unable to create socket");
 		snprintf(ping->ping_status, 50, "down");
-		free(new_hostname);
 		if (tcp_socket != -1) close(tcp_socket);
 		return HOST_DOWN;
 	}
@@ -992,32 +964,101 @@ int init_sockaddr(struct sockaddr_in *name, const char *hostname, unsigned short
 	}
 }
 
-/*! \fn char *remove_tcp_udp_from_hostname(char *hostname)
- *  \brief removes 'TCP[6]:' or 'UDP[6]:' from a hostname required to ping
+/*! \fn name_t *get_namebyhost(char *hostname, name_t *name)
+ *  \brief splits the hostname into method, name and port
  *
- *  \return char hostname a trimmed hostname
+ *  \return name_t containing a trimmed hostname, port, and optional method
  *
  */
-char *remove_tcp_udp_from_hostname(char *hostname) {
-	char *cleaned_hostname;
-
-	if (!(cleaned_hostname = (char *) malloc(strlen(hostname)+1))) {
-		die("ERROR: Fatal malloc error: ping.c remove_tcp_udp_from_hostname");
+name_t *get_namebyhost(char *hostname, name_t *name) {
+	if (name == NULL) {
+		SPINE_LOG_DEBUG(("get_namebyhost(%s) - Allocating name_t", hostname));
+		if (!(name = (name_t *) malloc(sizeof(name_t)))) {
+			die("ERROR: Fatal malloc error: ping.c get_namebyhost->name");
+		}
+		memset(name, '\0', sizeof(name_t));
 	}
 
-	if (!strncasecmp(hostname, "TCP:", 4) ||
-		!strncasecmp(hostname, "UDP:", 4)) {
-		memcpy(cleaned_hostname, hostname+4, strlen(hostname)-4);
-		cleaned_hostname[strlen(hostname)-4] = '\0';
-	} else if (!strncasecmp(hostname, "TCP6:", 5) ||
-        !strncasecmp(hostname, "UDP6:", 5)) {
-        memcpy(cleaned_hostname, hostname+5, strlen(hostname)-5);
-        cleaned_hostname[strlen(hostname)-5] = '\0';
-	} else {
-		strcpy(cleaned_hostname, hostname);
+	int tokens = 0;
+	char *stack = NULL;
+	char *token = NULL;
+
+	if (!(stack = (char *) malloc(strlen(hostname)+1))) {
+		die("ERROR: Fatal malloc error: ping.c get_namebyhost->stack");
+	}
+	memset(stack, '\0', strlen(hostname)+1);
+	strncopy(stack, hostname, strlen(stack));
+	token = strtok(stack, ":");
+
+	if (token == NULL) {
+		SPINE_LOG_DEBUG(("get_namebyhost(%s) - No delimiter, assume full hostname", hostname));
+		strncopy(name->hostname, hostname, SMALL_BUFSIZE);
 	}
 
-	return(cleaned_hostname);
+	while (token != NULL && tokens <= 3) {
+		tokens++;
+		SPINE_LOG_DEBUG(("get_namebyhost(%s) - Token #%i - %s", hostname, tokens, token));
+		if (tokens == 1) {
+			if (strlen(token) && token[0] == '[') {
+				SPINE_LOG_DEBUG(("get_namebyhost(%s) - Have TCPv6 method", hostname));
+				strncpy(name->hostname, hostname, sizeof(name->hostname));
+				break;
+			} else if (strlen(token) == 3) {
+				if (strncasecmp(token, "TCP", 3)) {
+					SPINE_LOG_DEBUG(("get_namebyhost(%s) - Have TCPv4 method", hostname));
+					name->method = 1;
+				} else if (strncasecmp(hostname, "UDP", 3)) {
+					SPINE_LOG_DEBUG(("get_namebyhost(%s) - Have UDPv4 method", hostname));
+					name->method = 2;
+				} else {
+					SPINE_LOG_DEBUG(("get_namebyhost(%s) - No matching method for 3 chars: %s", hostname, token));
+					// assume we have had a method
+					tokens++;
+				}
+			} else if (strlen(token) == 4) {
+				if (strncasecmp(token, "TCP6", 3)) {
+					SPINE_LOG_DEBUG(("get_namebyhost(%s) - Have TCPv6 method", hostname));
+					name->method = 3;
+				} else if (strncasecmp(hostname, "UDP6", 3)) {
+					SPINE_LOG_DEBUG(("get_namebyhost(%s) - Have UDPv6 method", hostname));
+					name->method = 4;
+				} else {
+					SPINE_LOG_DEBUG(("get_namebyhost(%s) - No matching method for 4 chars: %s", hostname, token));
+
+					// assume we have had a method
+					tokens++;
+				}
+			} else {
+				SPINE_LOG_DEBUG(("get_hostbyname(%s) - No matching method for %li chars: %s", hostname, strlen(token), token));
+
+				// assume we have had a method
+				tokens++;
+			}
+		}
+
+		if (tokens == 2) {
+			SPINE_LOG_DEBUG(("get_namebyhost(%s) - Setting hostname: %s", hostname, token));
+			strncpy(name->hostname, token, sizeof(name->hostname));
+			name->hostname[strlen(token)] = '\0';
+		}
+
+		if (tokens == 3 && strlen(token)) {
+			SPINE_LOG_DEBUG(("get_namebyhost(%s) - Setting port: %s", hostname, token));
+			name->port = atoi(token);
+		}
+
+		if (tokens > 3) {
+			SPINE_LOG_DEBUG(("get_namebyhost(%s) - Unexpected token: %i", hostname, tokens));
+		}
+		token = strtok(NULL, ":");
+	}
+
+	if (stack != NULL) {
+		free(stack);
+		stack = NULL;
+	}
+
+	return name;
 }
 
 /*! \fn unsigned short int get_checksum(void* buf, int len)
@@ -1070,18 +1111,7 @@ void update_host_status(int status, host_t *host, ping_t *ping, int availability
  	double hundred_percent = 100.00;
 	char   current_date[40];
 
-	time_t nowbin;
-	struct tm now_time;
-	struct tm *now_ptr;
-
-	/* get time for poller_output table */
-	if (time(&nowbin) == (time_t) - 1) {
-		die("ERROR: Could not get time of day from time()");
-	}
-	localtime_r(&nowbin,&now_time);
-	now_ptr = &now_time;
-
-	strftime(current_date, 40, "%Y-%m-%d %H:%M", now_ptr);
+	snprintf(current_date, 40, "%lu", time(NULL));
 
 	/* host is down */
 	if (status == HOST_DOWN) {
@@ -1097,7 +1127,7 @@ void update_host_status(int status, host_t *host, ping_t *ping, int availability
 			if ((strlen(host->snmp_community) == 0) && (host->snmp_version < 3)) {
 				snprintf(host->status_last_error, BUFSIZE, "%s", ping->ping_response);
 			} else {
-				snprintf(host->status_last_error, BUFSIZE,"%s, %s",ping->snmp_response,ping->ping_response);
+				snprintf(host->status_last_error, BUFSIZE, "%s, %s", ping->snmp_response, ping->ping_response);
 			}
 			break;
 		case AVAIL_SNMP:
