@@ -1,6 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
@@ -194,31 +195,67 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 		session.community     = (unsigned char*) snmp_community;
 		session.community_len = strlen(snmp_community);
 	} else {
-		/* set the SNMPv3 user name */
-		session.securityName         = snmp_username;
-		session.securityNameLen      = strlen(session.securityName);
+		session.securityName    = snmp_username;
+		session.securityNameLen = strlen(session.securityName);
 
 		if (snmp_context && strlen(snmp_context)) {
-			session.contextName          = snmp_context;
-			session.contextNameLen       = strlen(session.contextName);
+			session.contextName    = snmp_context;
+			session.contextNameLen = strlen(session.contextName);
 		}
 
 		if (snmp_engine_id && strlen(snmp_engine_id)) {
-			session.contextEngineID      = (unsigned char*) snmp_engine_id;
-			session.contextEngineIDLen   = strlen(snmp_engine_id);
+			session.contextEngineID    = (unsigned char*) snmp_engine_id;
+			session.contextEngineIDLen = strlen(snmp_engine_id);
 		}
 
-		session.securityAuthKeyLen   = USM_AUTH_KU_LEN;
+		session.securityAuthKeyLen = USM_AUTH_KU_LEN;
 
 		/* set the authentication protocol */
 		if (strcmp(snmp_auth_protocol, "MD5") == 0) {
+			#ifndef NETSNMP_DISABLE_MD5
 			/* set the authentication method to MD5 */
 			session.securityAuthProto    = snmp_duplicate_objid(usmHMACMD5AuthProtocol, USM_AUTH_PROTO_MD5_LEN);
 			session.securityAuthProtoLen = USM_AUTH_PROTO_MD5_LEN;
+			#else
+			SPINE_LOG(("SNMP: Error MD5 is no longer supported on this system."));
+			return 0;
+			#endif
 		} else if (strcmp(snmp_auth_protocol, "SHA") == 0) {
-			/* set the authentication method to SHA1 */
 			session.securityAuthProto    = snmp_duplicate_objid(usmHMACSHA1AuthProtocol, USM_AUTH_PROTO_SHA_LEN);
 			session.securityAuthProtoLen = USM_AUTH_PROTO_SHA_LEN;
+		} else if (strcmp(snmp_auth_protocol, "SHA224") == 0) {
+			#if defined(HAVE_EVP_SHA224) && defined(usmHMAC128SHA224AuthProtocol)
+			session.securityAuthProto    = snmp_duplicate_objid(usmHMAC128SHA224AuthProtocol, OID_LENGTH(usmHMAC128SHA224AuthProtocol));
+			session.securityAuthProtoLen = OID_LENGTH(usmHMAC128SHA224AuthProtocol);
+			#else
+			SPINE_LOG(("SNMP: Error SHA224 is not supported on this system.  Upgrade the Net-SNMP API to 5.8+"));
+			return 0;
+			#endif
+		} else if (strcmp(snmp_auth_protocol, "SHA256") == 0) {
+			//#if defined(HAVE_EVP_SHA224) && defined(usmHMAC192SHA256AuthProtocol)
+			#if defined(HAVE_EVP_SHA224) && defined(usmHMAC192SHA256AuthProtocol)
+			session.securityAuthProto    = snmp_duplicate_objid(usmHMAC192SHA256AuthProtocol, OID_LENGTH(usmHMAC192SHA256AuthProtocol));
+			session.securityAuthProtoLen = OID_LENGTH(usmHMAC192SHA256AuthProtocol);
+			#else
+			SPINE_LOG(("SNMP: Error SHA256 is not supported on this system.  Upgrade the Net-SNMP API to 5.8+"));
+			return 0;
+			#endif
+		} else if (strcmp(snmp_auth_protocol, "SHA384") == 0) {
+			#if defined(HAVE_EVP_SHA384) && defined(usmHMAC256SHA384AuthProtocol)
+			session.securityAuthProto    = snmp_duplicate_objid(usmHMAC256SHA384AuthProtocol, OID_LENGTH(usmHMAC256SHA384AuthProtocol));
+			session.securityAuthProtoLen = USM_HMAC256SHA384_AUTH_LEN;
+			#else
+			SPINE_LOG(("SNMP: Error SHA384 is not supported on this system.  Upgrade the Net-SNMP API to 5.8+"));
+			return 0;
+			#endif
+		} else if (strcmp(snmp_auth_protocol, "SHA512") == 0) {
+			#if defined(HAVE_EVP_SHA384) && defined(usmHMAC384SHA512AuthProtocol)
+			session.securityAuthProto    = snmp_duplicate_objid(usmHMAC384SHA512AuthProtocol, OID_LENGTH(usmHMAC384SHA512AuthProtocol));
+			session.securityAuthProtoLen = USM_HMAC384SHA512_AUTH_LEN;
+			#else
+			SPINE_LOG(("SNMP: Error SHA512 is not supported on this system.  Upgrade the Net-SNMP API to 5.8+"));
+			return 0;
+			#endif
 		}
 
 		if (strlen(snmp_password)) {
@@ -228,15 +265,15 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 				(u_char *) snmp_password,
 				strlen(snmp_password),
 				session.securityAuthKey,
-				&(session.securityAuthKeyLen)) != SNMPERR_SUCCESS) {
+				&session.securityAuthKeyLen) != SNMPERR_SUCCESS) {
 				SPINE_LOG(("SNMP: Error generating SNMPv3 Ku from authentication pass phrase."));
 			}
 		}
 
 		/* set the privacy protocol to none */
 		if (strcmp(snmp_priv_protocol, "[None]") == 0 || (strlen(snmp_priv_passphrase) == 0)) {
-			session.securityPrivProto    = snmp_duplicate_objid(usmNoPrivProtocol, OIDSIZE(usmNoPrivProtocol));
-			session.securityPrivProtoLen = OIDSIZE(usmNoPrivProtocol);
+			session.securityPrivProto    = snmp_duplicate_objid(usmNoPrivProtocol, OID_LENGTH(usmNoPrivProtocol));
+			session.securityPrivProtoLen = OID_LENGTH(usmNoPrivProtocol);
 			session.securityPrivKeyLen   = USM_PRIV_KU_LEN;
 
 			/* set the security level to authenticate, but not encrypted */
@@ -247,25 +284,47 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 			}
 		} else {
 			if (strcmp(snmp_priv_protocol, "DES") == 0) {
+				#if defined(USM_PRIV_PROTO_DES_LEN) && !defined(NETSNMP_DISABLE_DES)
 				session.securityPrivProto    = snmp_duplicate_objid(usmDESPrivProtocol, USM_PRIV_PROTO_DES_LEN);
 				session.securityPrivProtoLen = USM_PRIV_PROTO_DES_LEN;
 				session.securityPrivKeyLen   = USM_PRIV_KU_LEN;
-
-				/* set the security level to authenticate, and encrypted */
 				session.securityLevel        = SNMP_SEC_LEVEL_AUTHPRIV;
-			} else {
+				#else
+				SPINE_LOG(("SNMP: Error DES is no longer supported on this system"));
+				return 0;
+				#endif
+			} else if (strcmp(snmp_priv_protocol, "AES128") == 0) {
 				#if defined(USM_PRIV_PROTO_AES_LEN)
 				session.securityPrivProto    = snmp_duplicate_objid(usmAESPrivProtocol, USM_PRIV_PROTO_AES_LEN);
 				session.securityPrivProtoLen = USM_PRIV_PROTO_AES_LEN;
 				session.securityPrivKeyLen   = USM_PRIV_KU_LEN;
-				#else
-				session.securityPrivProto    = snmp_duplicate_objid(usmAES128PrivProtocol, OIDSIZE(usmAES128PrivProtocol));
-				session.securityPrivProtoLen = OIDSIZE(usmAES128PrivProtocol);
-				session.securityPrivKeyLen   = USM_PRIV_KU_LEN;
-				#endif
-
-				/* set the security level to authenticate, and encrypted */
 				session.securityLevel        = SNMP_SEC_LEVEL_AUTHPRIV;
+				#else
+				session.securityPrivProto    = snmp_duplicate_objid(usmAES128PrivProtocol, OID_LENGTH(usmAES128PrivProtocol));
+				session.securityPrivProtoLen = OID_LENGTH(usmAES128PrivProtocol);
+				session.securityPrivKeyLen   = USM_PRIV_KU_LEN;
+				session.securityLevel        = SNMP_SEC_LEVEL_AUTHPRIV;
+				#endif
+			} else if(strcmp(snmp_priv_protocol, "AES192") == 0) {
+				#if defined(NETSNMP_DRAFT_BLUMENTHAL_AES_04) && defined(usmAES192PrivProtocol)
+				session.securityPrivProto    = snmp_duplicate_objid(usmAES192PrivProtocol, OID_LENGTH(usmAES192PrivProtocol));
+				session.securityPrivProtoLen = OID_LENGTH(usmAES192PrivProtocol);
+				session.securityPrivKeyLen   = BYTESIZE(SNMP_TRANS_PRIVLEN_AES192);
+				session.securityLevel        = SNMP_SEC_LEVEL_AUTHPRIV;
+				#else
+				SPINE_LOG(("SNMP: Error AES-192 is not supported in the Net-SNMP API, upgrade the Net-SNMP libraries."));
+				return 0;
+				#endif
+			} else if(strcmp(snmp_priv_protocol, "AES256") == 0) {
+				#if defined(NETSNMP_DRAFT_BLUMENTHAL_AES_04) && defined(usmAES256PrivProtocol)
+				session.securityPrivProto    = snmp_duplicate_objid(usmAES256PrivProtocol, OID_LENGTH(usmAES256PrivProtocol));
+				session.securityPrivProtoLen = OID_LENGTH(usmAES256PrivProtocol);
+				session.securityPrivKeyLen   = BYTESIZE(SNMP_TRANS_PRIVLEN_AES256);
+				session.securityLevel        = SNMP_SEC_LEVEL_AUTHPRIV;
+				#else
+				SPINE_LOG(("SNMP: Error AES-256 is not supported in the Net-SNMP API, upgrade the Net-SNMP libraries."));
+				return 0;
+				#endif
 			}
 
 			/* set the privacy key to the hashed version. */
@@ -274,8 +333,9 @@ void *snmp_host_init(int host_id, char *hostname, int snmp_version, char *snmp_c
 				(u_char *) snmp_priv_passphrase,
 				strlen(snmp_priv_passphrase),
 				session.securityPrivKey,
-				&(session.securityPrivKeyLen)) != SNMPERR_SUCCESS) {
+				&session.securityPrivKeyLen) != SNMPERR_SUCCESS) {
 				SPINE_LOG(("SNMP: Error generating SNMPv3 Ku from privacy pass phrase."));
+				return 0;
 			}
 		}
 	}
@@ -384,11 +444,11 @@ char *snmp_get(host_t *current_host, char *snmp_oid) {
 
 					snprintf(result_string, RESULTS_BUFFER, "%s", trim(temp_result));
 				} else {
-					SPINE_LOG_HIGH(("ERROR: Failed to get oid '%s' for Device[%d]",  snmp_oid, current_host->id));
+					SPINE_LOG_HIGH(("ERROR: Failed to get oid '%s' for Device[%d] with Response[%ld]",  snmp_oid, current_host->id, response->errstat));
 				}
 			}
 		} else {
-			SPINE_LOG_HIGH(("ERROR: Failed to get oid '%s' for Device[%d]",  snmp_oid, current_host->id));
+			SPINE_LOG_HIGH(("ERROR: Failed to get oid '%s' for Device[%d] with Status[%d]",  snmp_oid, current_host->id, status));
 		}
 
 		if (response) {
@@ -516,9 +576,9 @@ int snmp_count(host_t *current_host, char *snmp_oid) {
 	status = STAT_DESCRIP_ERROR;
 
 	if (is_debug_device(current_host->id)) {
-		SPINE_LOG(("NOTE: walk starts at OID %s", snmp_oid));
+		SPINE_LOG(("DEBUG: walk starts at OID %s", snmp_oid));
 	} else {
-		SPINE_LOG_DEBUG(("NOTE: walk starts at OID %s", snmp_oid));
+		SPINE_LOG_DEBUG(("DEBUG: walk starts at OID %s", snmp_oid));
 	}
 
 	if (current_host->snmp_session != NULL) {
