@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -140,6 +140,7 @@
 #define MEGA_BUFSIZE 1024000
 #define HUGE_BUFSIZE 2048000
 #define LOGSIZE 65535
+#define LRG_LOGSIZE 1024000
 #define BITSINBYTE 8
 #define THIRTYTWO 4294967295ul
 #define SIXTYFOUR 18446744073709551615ul
@@ -170,6 +171,7 @@
 #define LOCK_SNMP 0
 #define LOCK_SETEUID 2
 #define LOCK_GHBN 3
+#define LOCK_POOL 4
 #define LOCK_SYSLOG 5
 #define LOCK_PHP 6
 #define LOCK_PHP_PROC_0 7
@@ -182,11 +184,17 @@
 #define LOCK_PHP_PROC_7 14
 #define LOCK_PHP_PROC_8 15
 #define LOCK_PHP_PROC_9 16
-#define LOCK_PEND 17
+#define LOCK_PHP_PROC_10 17
+#define LOCK_PHP_PROC_11 18
+#define LOCK_PHP_PROC_12 19
+#define LOCK_PHP_PROC_13 20
+#define LOCK_PHP_PROC_14 21
+#define LOCK_THDET 40
 
 #define LOCK_SNMP_O 0
 #define LOCK_SETEUID_O 2
 #define LOCK_GHBN_O 3
+#define LOCK_POOL_O 4
 #define LOCK_SYSLOG_O 5
 #define LOCK_PHP_O 6
 #define LOCK_PHP_PROC_0_O 7
@@ -199,7 +207,12 @@
 #define LOCK_PHP_PROC_7_O 14
 #define LOCK_PHP_PROC_8_O 15
 #define LOCK_PHP_PROC_9_O 16
-#define LOCK_PEND_O 17
+#define LOCK_PHP_PROC_10_O 17
+#define LOCK_PHP_PROC_11_O 18
+#define LOCK_PHP_PROC_12_O 19
+#define LOCK_PHP_PROC_13_O 20
+#define LOCK_PHP_PROC_14_O 21
+#define LOCK_THDET_O 40
 
 /* poller actions */
 #define POLLER_ACTION_SNMP 0
@@ -273,7 +286,7 @@
 #define ICMP_HDR_SIZE 8
 
 /* required for PHP Script Server */
-#define MAX_PHP_SERVERS 10
+#define MAX_PHP_SERVERS 15
 #define PHP_READY 0
 #define PHP_BUSY 1
 #define PHP_INIT 999
@@ -306,9 +319,6 @@
 #define STRMATCH(a,b)	(strcmp((a),(b)) == 0)
 #define STRIMATCH(a,b)	(strcasecmp((a),(b)) == 0)
 
-/* convert timeval to double (rounding to the nearest microsecond) */
-#define TIMEVAL_TO_DOUBLE(tv)	( (tv).tv_sec + ((double) (tv).tv_usec / 1000000) )
-
 /* When any kind of poller wants to set an undefined value; this particular
  * value used ('U') springs from the requirements of rrdupdate. We also
  * include the corresponding test macro which looks for the literal string
@@ -336,6 +346,7 @@ typedef struct config_struct {
 	int    parent_fork;
 	int    num_parent_processes;
 	int    script_timeout;
+	int    active_profiles;
 	int    threads;
 	int    logfile_processed;
 	int    boost_enabled;
@@ -361,7 +372,7 @@ typedef struct config_struct {
 	unsigned int db_port;
 	int    dbversion;
 	/* path information */
-	char   path_logfile[BUFSIZE];
+	char   path_logfile[DBL_BUFSIZE];
 	char   path_php[BUFSIZE];
 	char   path_php_server[BUFSIZE];
 	/* logging options */
@@ -429,7 +440,7 @@ typedef struct target_struct {
 	int    snmp_version;
 	char   snmp_username[50];
 	char   snmp_password[50];
-	char   snmp_auth_protocol[5];
+	char   snmp_auth_protocol[7];
 	char   snmp_priv_passphrase[200];
 	char   snmp_priv_protocol[7];
 	char   snmp_context[65];
@@ -466,10 +477,12 @@ typedef struct snmp_oids {
  *
  */
 typedef struct poller_thread {
+	int device_counter;
 	int host_id;
 	int host_thread;
-	int last_host_thread;
+	int host_threads;
 	int host_data_ids;
+	int threads_complete;
 	int complete;
 	char *host_time;
 	double host_time_double;
@@ -502,7 +515,7 @@ typedef struct host_struct {
 	int    snmp_version;
 	char   snmp_username[50];
 	char   snmp_password[50];
-	char   snmp_auth_protocol[5];
+	char   snmp_auth_protocol[7];
 	char   snmp_priv_passphrase[200];
 	char   snmp_priv_protocol[7];
 	char   snmp_context[65];
@@ -512,7 +525,7 @@ typedef struct host_struct {
 	int    snmp_retries;
 	char   snmp_sysDescr[600];
 	char   snmp_sysObjectID[160];
-	u_int  snmp_sysUpTimeInstance;
+	unsigned long long snmp_sysUpTimeInstance;
 	char   snmp_sysContact[300];
 	char   snmp_sysName[300];
 	char   snmp_sysLocation[600];
@@ -552,7 +565,7 @@ typedef struct host_reindex_struct {
 	int    action;
 } reindex_t;
 
-/*! Ping Result Struction
+/*! Ping Result Structure
  *
  * This structure holds the results of a host ping.
  *
@@ -565,6 +578,28 @@ typedef struct ping_results {
 	char   snmp_response[SMALL_BUFSIZE];
 } ping_t;
 
+/*! Name Result Structure
+ *
+ * This structure holds the results of a name/port split
+ *
+ */
+typedef struct name_port {
+	// Method = 0 - default, 1 - tcp, 2 - udp
+	int	method;
+	char	hostname[SMALL_BUFSIZE];
+	int	port;
+} name_t;
+
+/*! MySQL Connection Pool Structure
+ *
+ * This structure holds the mysql connetion pool object.
+ */
+typedef struct db_connection {
+	int   id;
+	int   free;
+	MYSQL mysql;
+} pool_t;
+
 /* Include all Standard Spine Headers */
 #include "poller.h"
 #include "locks.h"
@@ -582,6 +617,9 @@ extern config_t set;
 extern php_t  *php_processes;
 extern char   start_datetime[20];
 extern char   config_paths[CONFIG_PATHS][BUFSIZE];
-extern sem_t  active_threads;
+extern sem_t  available_threads;
+extern sem_t  available_scripts;
+extern pool_t *db_pool_remote;
+extern pool_t *db_pool_local;
 
 #endif /* not _SPINE_H_ */
