@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -119,6 +119,36 @@ static const char *getsetting(MYSQL *psql, int mode, const char *setting) {
 	}
 }
 
+/*! \fn int putsetting(MYSQL *psql, const char *setting, const char *value)
+ *  \brief Set's a specific Cacti setting.
+ *
+ *  Given a pointer to a database and the name of a setting, and value of that setting
+ *  set the Cacti setting in the database to the value.
+ *
+ *  \return true for sucessful or false for failed
+ *
+ */
+int putsetting(MYSQL *psql, int mode, const char *mysetting, const char *myvalue) {
+	char  qstring[512];
+	int   result = 0;
+
+	assert(psql    != 0);
+	assert(mysetting != 0);
+	assert(myvalue   != 0);
+
+	sprintf(qstring, "INSERT INTO settings (name, value) "
+		"VALUES ('%s', '%s') "
+		"ON DUPLICATE KEY UPDATE value=VALUES(value)", mysetting, myvalue);
+
+	result = db_insert(psql, mode, qstring);
+
+	if (result == 0) {
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
 /*! \fn static const char *getpsetting(MYSQL *psql, const char *setting)
  *  \brief Returns a character pointer to a Cacti poller setting.
  *
@@ -304,6 +334,10 @@ void read_config_options() {
 	char       web_root[BUFSIZE];
 	char       sqlbuf[SMALL_BUFSIZE], *sqlp = sqlbuf;
 	const char *res;
+	char       spine_capabilities[SMALL_BUFSIZE];
+
+	/* publish spine snmpv3 capabilities to the database */
+	memset(spine_capabilities, 0, sizeof(spine_capabilities));
 
 	db_connect(LOCAL, &mysql);
 
@@ -338,17 +372,17 @@ void read_config_options() {
 	/* determine logfile path */
 	if ((res = getsetting(&mysql, LOCAL, "path_cactilog")) != 0) {
 		if (strlen(res) != 0) {
-			snprintf(set.path_logfile, SMALL_BUFSIZE, "%s", res);
+			snprintf(set.path_logfile, DBL_BUFSIZE, "%s", res);
 		} else {
 			if (strlen(web_root) != 0) {
-				snprintf(set.path_logfile, SMALL_BUFSIZE, "%s/log/cacti.log", web_root);
+				snprintf(set.path_logfile, DBL_BUFSIZE, "%s/log/cacti.log", web_root);
 			} else {
 				set.path_logfile[0] ='\0';
 			}
 		}
 		free((char *)res);
 	} else {
-		snprintf(set.path_logfile, SMALL_BUFSIZE, "%s/log/cacti.log", web_root);
+		snprintf(set.path_logfile, DBL_BUFSIZE, "%s/log/cacti.log", web_root);
  	}
 
 	/* get log separator */
@@ -371,12 +405,6 @@ void read_config_options() {
 		}
 	}
 
-	/* log the path_webroot variable */
-	SPINE_LOG_DEBUG(("DEBUG: The path_php_server variable is %s", set.path_php_server));
-
-	/* log the path_cactilog variable */
-	SPINE_LOG_DEBUG(("DEBUG: The path_cactilog variable is %s", set.path_logfile));
-
 	/* determine log file, syslog or both, default is 1 or log file only */
 	if ((res = getsetting(&mysql, LOCAL, "log_destination")) != 0) {
 		set.log_destination = parse_logdest(res, LOGDEST_FILE);
@@ -385,10 +413,17 @@ void read_config_options() {
 		set.log_destination = LOGDEST_FILE;
 	}
 
+	/* log the path_webroot variable */
+	SPINE_LOG_DEBUG(("DEBUG: The path_php_server variable is %s", set.path_php_server));
+
+	/* log the path_cactilog variable */
+	SPINE_LOG_DEBUG(("DEBUG: The path_cactilog variable is %s", set.path_logfile));
+
 	/* log the log_destination variable */
 	SPINE_LOG_DEBUG(("DEBUG: The log_destination variable is %i (%s)",
-			set.log_destination,
-			printable_logdest(set.log_destination)));
+		set.log_destination,
+		printable_logdest(set.log_destination)));
+
 	set.logfile_processed = TRUE;
 
 	/* get PHP Path Information for Scripting */
@@ -586,6 +621,21 @@ void read_config_options() {
 	/* log the script timeout value */
 	SPINE_LOG_DEBUG(("DEBUG: The number of php script servers to run is %i", set.php_servers));
 
+	/* get the number of script server processes to run */
+	if ((res = getsetting(&mysql, LOCAL, "active_profiles")) != 0) {
+		set.active_profiles = atoi(res);
+		free((char *)res);
+
+		if (set.active_profiles <= 0) {
+			set.active_profiles = 0;
+		}
+	} else {
+		set.active_profiles = 0;
+	}
+
+	/* log the script timeout value */
+	SPINE_LOG_DEBUG(("DEBUG: The number of active data source profiles is %i", set.active_profiles));
+
 	/*----------------------------------------------------------------
 	 * determine if the php script server is required by searching for
 	 * all the host records for an action of POLLER_ACTION_PHP_SCRIPT_SERVER.
@@ -657,6 +707,40 @@ void read_config_options() {
 	/* log the snmp_max_get_size variable */
 	SPINE_LOG_DEBUG(("DEBUG: The Maximum SNMP OID Get Size is %i", set.snmp_max_get_size));
 
+	strcat(spine_capabilities, "{ authProtocols: \"");
+	#ifndef NETSNMP_DISABLE_MD5
+	strcat(spine_capabilities, "MD5,");
+	#endif
+
+	strcat(spine_capabilities, "SHA");
+
+	#if defined(HAVE_EVP_SHA224) && defined(usmHMAC192SHA256AuthProtocol)
+	strcat(spine_capabilities, ",SHA224,SHA256");
+	#endif
+	#if defined(HAVE_EVP_SHA384) && defined(usmHMAC384SHA512AuthProtocol)
+	strcat(spine_capabilities, ",SHA384,SHA512");
+	#endif
+	strcat(spine_capabilities, "\"");
+
+	strcat(spine_capabilities, ", privProtocols: \"");
+	#ifndef NETSNMP_DISABLE_DES
+	strcat(spine_capabilities, "DES");
+	#endif
+	#ifdef HAVE_AES
+	strcat(spine_capabilities, ",AES128");
+	#if defined(NETSNMP_DRAFT_BLUMENTHAL_AES_04) && defined(usmAES192PrivProtocol)
+	strcat(spine_capabilities, ",AES192");
+	#endif
+	#if defined(NETSNMP_DRAFT_BLUMENTHAL_AES_04) && defined(usmAES256PrivProtocol)
+	strcat(spine_capabilities, ",AES256");
+	#endif
+	#endif
+	strcat(spine_capabilities, "\" }");
+
+	if (set.poller_id == 1) {
+		putsetting(&mysql, LOCAL, "spine_capabilities", spine_capabilities);
+	}
+
 	db_disconnect(&mysql);
 
 	if (set.poller_id > 1 && set.mode == REMOTE_ONLINE) {
@@ -673,7 +757,7 @@ void poller_push_data_to_main() {
 	int        rows;
 	char       sqlbuf[MEGA_BUFSIZE];
 	char       *sqlp = sqlbuf;
-	char       query[BUFSIZE];
+	char       query[MEGA_BUFSIZE];
 	char       prefix[BUFSIZE];
 	char       suffix[BUFSIZE];
 	char       tmpstr[SMALL_BUFSIZE];
@@ -689,13 +773,24 @@ void poller_push_data_to_main() {
 
 	SPINE_LOG_MEDIUM(("Pushing Host Status to Main Server"));
 
-	snprintf(query, BUFSIZE, "SELECT id, snmp_sysDescr, snmp_sysObjectID, "
-		"snmp_sysUpTimeInstance, snmp_sysContact, snmp_sysName, snmp_sysLocation, "
-		"status, status_event_count, status_fail_date, status_rec_date, "
-		"status_last_error, min_time, max_time, cur_time, avg_time, polling_time, "
-		"total_polls, failed_polls, availability, last_updated "
-		"FROM host "
-		"WHERE poller_id = %d", set.poller_id);
+	if (strlen(set.host_id_list)) {
+		snprintf(query, MEGA_BUFSIZE, "SELECT id, snmp_sysDescr, snmp_sysObjectID, "
+			"snmp_sysUpTimeInstance, snmp_sysContact, snmp_sysName, snmp_sysLocation, "
+			"status, status_event_count, status_fail_date, status_rec_date, "
+			"status_last_error, min_time, max_time, cur_time, avg_time, polling_time, "
+			"total_polls, failed_polls, availability, last_updated "
+			"FROM host "
+			"WHERE poller_id = %d "
+			"AND id IN (%s)", set.poller_id, set.host_id_list);
+	} else {
+		snprintf(query, MEGA_BUFSIZE, "SELECT id, snmp_sysDescr, snmp_sysObjectID, "
+			"snmp_sysUpTimeInstance, snmp_sysContact, snmp_sysName, snmp_sysLocation, "
+			"status, status_event_count, status_fail_date, status_rec_date, "
+			"status_last_error, min_time, max_time, cur_time, avg_time, polling_time, "
+			"total_polls, failed_polls, availability, last_updated "
+			"FROM host "
+			"WHERE poller_id = %d", set.poller_id);
+	}
 
 	snprintf(prefix, BUFSIZE, "INSERT INTO host (id, snmp_sysDescr, snmp_sysObjectID, "
 		"snmp_sysUpTimeInstance, snmp_sysContact, snmp_sysName, snmp_sysLocation, "
@@ -800,9 +895,17 @@ void poller_push_data_to_main() {
 
 	SPINE_LOG_MEDIUM(("Pushing Poller Item RRD Next Step to Main Server"));
 
-	snprintf(query, BUFSIZE, "SELECT local_data_id, host_id, rrd_name, rrd_step, rrd_next_step "
-		"FROM poller_item "
-		"WHERE poller_id = %i", set.poller_id);
+	if (strlen(set.host_id_list)) {
+		snprintf(query, MEGA_BUFSIZE, "SELECT local_data_id, host_id, rrd_name, rrd_step, rrd_next_step "
+			"FROM poller_item "
+			"WHERE poller_id = %d "
+			"AND host_id IN (%s)", set.poller_id, set.host_id_list);
+	} else {
+		snprintf(query, MEGA_BUFSIZE, "SELECT local_data_id, host_id, rrd_name, rrd_step, rrd_next_step "
+			"FROM poller_item "
+			"WHERE poller_id = %d ",
+			set.poller_id);
+	}
 
 	snprintf(prefix, BUFSIZE, "INSERT INTO poller_item (local_data_id, host_id, rrd_name, rrd_step, rrd_next_step) VALUES ");
 
@@ -974,7 +1077,7 @@ void config_defaults() {
 void die(const char *format, ...) {
 	va_list	args;
 	char logmessage[BUFSIZE];
-	char flogmessage[BUFSIZE];
+	char flogmessage[DBL_BUFSIZE];
 	int old_errno = errno;
 
 	va_start(args, format);
@@ -989,29 +1092,15 @@ void die(const char *format, ...) {
 
 	if (set.logfile_processed) {
 		if (set.parent_fork == SPINE_PARENT) {
-			snprintf(flogmessage, BUFSIZE, "%s (Spine parent)", logmessage);
+			snprintf(flogmessage, DBL_BUFSIZE, "%s (Spine parent)", logmessage);
 		} else {
-			snprintf(flogmessage, BUFSIZE, "%s (Spine thread)", logmessage);
+			snprintf(flogmessage, DBL_BUFSIZE, "%s (Spine thread)", logmessage);
 		}
 	} else {
-		snprintf(flogmessage, BUFSIZE, "%s (Spine init)", logmessage);
+		snprintf(flogmessage, DBL_BUFSIZE, "%s (Spine init)", logmessage);
 	}
 
-	SPINE_LOG(("%s", flogmessage));
-
-#ifdef HAS_EXECINFO_H
-	printf("Generating backtrace...%ld line(s)...\n", set.exit_size);
-	if (set.exit_size) {
-		char **exit_strings = backtrace_symbols(set.exit_stack, set.exit_size);
-		if (exit_strings) {
-			int row = 0;
-			for (row = 0; row < set.exit_size; row++)
-			        printf("%3d: %s\n", row, exit_strings[row]);
-
-			free(exit_strings);
-		}
-	}
-#endif
+	fprintf(stderr, "%s", flogmessage);
 
 	if (set.parent_fork == SPINE_PARENT) {
 		if (set.php_initialized) {
@@ -1084,6 +1173,7 @@ int spine_log(const char *format, ...) {
 	time_t nowbin;
 	struct tm now_time;
 	struct tm *now_ptr;
+	struct timeval now;
 
 	/* keep track of an errored log file */
 	static int log_error = FALSE;
@@ -1091,6 +1181,9 @@ int spine_log(const char *format, ...) {
 	char logprefix[SMALL_BUFSIZE]; /* Formatted Log Prefix */
 	char ulogmessage[LOGSIZE];     /* Un-Formatted Log Message */
 	char flogmessage[LOGSIZE];     /* Formatted Log Message */
+	char stdoutmessage[LRG_LOGSIZE];   /* Message for stdout */
+
+	double cur_time;
 
 	va_start(args, format);
 	vsnprintf(ulogmessage, LOGSIZE - 1, format, args);
@@ -1099,13 +1192,8 @@ int spine_log(const char *format, ...) {
 	/* default for "console" messages to go to stdout */
 	fp = stdout;
 
-	if (IS_LOGGING_TO_STDOUT()) {
-		puts(trim(ulogmessage));
-		return TRUE;
-	}
-
 	/* log message prefix */
-	snprintf(logprefix, SMALL_BUFSIZE, "SPINE: Poller[%i] PID[%i] ", set.poller_id, getpid());
+	snprintf(logprefix, SMALL_BUFSIZE, "SPINE: Poller[%i] PID[%i] PT[%ld] ", set.poller_id, getpid(), pthread_self());
 
 	/* get time for poller_output table */
 	nowbin = time(&nowbin);
@@ -1113,6 +1201,13 @@ int spine_log(const char *format, ...) {
 	localtime_r(&nowbin,&now_time);
 	now_ptr = &now_time;
 
+	if (IS_LOGGING_TO_STDOUT()) {
+		cur_time = get_time_as_double();
+		sprintf(stdoutmessage, "Total[%3.4f] %s", cur_time - start_time, ulogmessage);
+		puts(stdoutmessage);
+		return TRUE;
+	}
+
 	char * log_fmt = get_date_format();
 	if (strlen(log_fmt) == 0) {
 		#ifdef DISABLE_STDERR
@@ -1471,27 +1566,24 @@ char *add_slashes(char *string) {
  *
  *  \return pointer to destination string
  *
- */
+*/
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-overflow"
 char *strncopy(char *dst, const char *src, size_t obuf) {
 	assert(dst != 0);
 	assert(src != 0);
 
 	size_t len;
 
-	len = strlen(src);
-
-	if (!len) {
-		dst[0] = '\0';
-	} else if (len < obuf) {
+	len = (strlen(src) < obuf) ? strlen(src) : obuf;
+	if (len) {
 		strncpy(dst, src, len);
-		dst[len] = '\0';
-	} else {
-		strncpy(dst, src, --obuf);
-		dst[obuf] = '\0';
 	}
 
+	dst[len] = '\0';
 	return dst;
 }
+#pragma GCC diagnostic pop
 
 /*! \fn double get_time_as_double()
  *  \brief fetches system time as a double-precison value
@@ -1503,7 +1595,7 @@ double get_time_as_double(void) {
 
 	gettimeofday(&now, NULL);
 
-	return TIMEVAL_TO_DOUBLE(now);
+	return (now).tv_sec + ((double) (now).tv_usec / 1000000);
 }
 
 /*! \fn string *get_host_poll_time()
@@ -1512,9 +1604,6 @@ double get_time_as_double(void) {
  *  \return host_time as a string
  */
 char *get_host_poll_time() {
-	time_t nowbin;
-	struct tm now_time;
-	struct tm *now_ptr;
 	char *host_time;
 
 	#define HOST_TIME_STRING_LEN 20
@@ -1524,16 +1613,7 @@ char *get_host_poll_time() {
 	}
 	host_time[0] = '\0';
 
-	/* get time for poller_output table */
-	if (time(&nowbin) == (time_t) - 1) {
-		die("ERROR: Could not get time of day from time() util.c get_host_poll_time()");
-	}
-	localtime_r(&nowbin,&now_time);
-	now_ptr = &now_time;
-
-	if (strftime(host_time, HOST_TIME_STRING_LEN, "%Y-%m-%d %H:%M:%S", now_ptr) == (size_t) 0) {
-		die("ERROR: Could not get string from strftime() util.c get_host_poll_time()");
-	}
+	sprintf(host_time, "%lu", (unsigned long) time(NULL));
 
 	return(host_time);
 }
@@ -1768,7 +1848,7 @@ void checkAsRoot() {
 
 	/* Add priviledge to send/receive ICMP packets */
 	if (priv_addset(privset, PRIV_NET_ICMPACCESS) < 0) {
-		SPINE_LOG_DEBUG(("Warning: Addition of PRIV_NET_ICMPACCESS to privset failed: '%s'.", strerror(errno)));
+		SPINE_LOG_DEBUG(("WARNING: Addition of PRIV_NET_ICMPACCESS to privset failed: '%s'.", strerror(errno)));
 	}
 
 	/* Compute the set of privileges that are never needed */
@@ -1777,12 +1857,12 @@ void checkAsRoot() {
 	/* Remove the set of unneeded privs from Permitted (and by
 	 * implication from Effective) */
 	if (setppriv(PRIV_OFF, PRIV_PERMITTED, privset) < 0) {
-		SPINE_LOG_DEBUG(("Warning: Dropping privileges from PRIV_PERMITTED failed: '%s'.", strerror(errno)));
+		SPINE_LOG_DEBUG(("WARNING: Dropping privileges from PRIV_PERMITTED failed: '%s'.", strerror(errno)));
 	}
 
 	/* Remove unneeded priv set from Limit to be safe */
 	if (setppriv(PRIV_OFF, PRIV_LIMIT, privset) < 0) {
-		SPINE_LOG_DEBUG(("Warning: Dropping privileges from PRIV_LIMIT failed: '%s'.", strerror(errno)));
+		SPINE_LOG_DEBUG(("WARNING: Dropping privileges from PRIV_LIMIT failed: '%s'.", strerror(errno)));
 	}
 
 	boolean_t pe = priv_ineffect(PRIV_NET_ICMPACCESS);
